
{Routines to "grow" species for the program WETLAND. Contains procedures
 GROWDRIV, GERMINATE, ANNGROW, PERGROW...}

{Copyright 1987 by Aaron M. Ellison}

{DEBUGGED AND RUNNING 03-MAY-1987}
{AMENDED}


PROCEDURE dispersal; {disperse seeds}

CONST
 alpha = 2; beta = 1;

VAR
 numseeds,r,mult,denom : real;
 bankyr,x,y,dymax,dymin,dxmax,dxmin : integer;
 bankstring : string[7];

BEGIN
 IF species[specno,8] = 'N ' THEN BEGIN
  IF (ycoord-3) > 0 THEN dymin := ycoord-3 ELSE dymin := 1;
  IF (ycoord+3) < 51 THEN dymax := ycoord+3 ELSE dymax := 50;
  IF (xcoord-3) > 0 THEN dxmin := xcoord-3 ELSE dxmin := 1;
  IF (xcoord+3) < 51 THEN dxmax := xcoord+3 ELSE dxmax := 50;
 END
 ELSE BEGIN
  IF (ycoord-7) > 0 THEN dymin := ycoord-7 ELSE dymin := 1;
  IF (ycoord+7) < 51 THEN dymax := ycoord+7 ELSE dymax := 50;
  IF (xcoord-7) > 0 THEN dxmin := xcoord-7 ELSE dxmin := 1;
  IF (xcoord+7) < 51 THEN dxmax := xcoord+7 ELSE dxmax := 50;
 END;
  bankstring := ' 010203';
  bankyr := Pos(species[specno,9],bankstring) div 2;
   IF species[specno,8] = 'N ' THEN mult := 0.5
   ELSE mult := 0.1;
   denom := exp(alpha*ln(beta))*factorial(alpha);
 FOR y := dymin to (ycoord-1) DO BEGIN
  {computes seed dispersal along y file "north" of source}
  r := ycoord-y; {calculates distance from seed source}
  numseeds := mult*(exp((alpha-1)*ln(r))*exp(r/beta)/denom);
   {determines number of seeds at distance r from a gamma distribution with
    parameters given by alpha, beta in the CONST definition above.}
  IF numseeds>1 THEN BEGIN
   j := (xcoord-1)*50 + y + (bankyr-1)*2500;
   xycell := Ptr(segadd+(j-1),offadd+8*(j-1));
   WITH xycell^ DO specbank[specno] := 1;
  END;
 END;
 FOR y := (ycoord+1) to dymax DO BEGIN
  {computes seed dispersal along y file "south" of source}
  r := y-ycoord; {calculates distance from source}
  numseeds := mult*(exp((alpha-1)*ln(r))*exp(r/beta)/denom);
   {determines number of seeds at distance r from a gamma distribution with
    parameters given by alpha, beta in the CONST definition above.}
  IF numseeds>1 THEN BEGIN
   j := (xcoord-1)*50 + y + (bankyr-1)*2500;
   xycell := Ptr(segadd + (j-1), offadd + 8*(j-1));
   WITH xycell^ DO specbank[specno] := 1;
  END;
 END;
 FOR x := dxmin to (xcoord-1) DO BEGIN
  {computes seed dispersal along x file "west" of source}
 r := xcoord-x;
  numseeds := mult*(exp((alpha-1)*ln(r))*exp(r/beta)/denom);
   {determines number of seeds at distance r from a gamma distribution with
    parameters given by alpha, beta in the CONST definition above.}
  IF numseeds>1 THEN BEGIN
   j := (x-1)*50 + ycoord + (bankyr-1)*2500;
   xycell := Ptr(segadd + (j-1), offadd + 8*(j-1));
   WITH xycell^ DO specbank[specno] := 1;
  END;
 END;
 FOR x := (xcoord+1) to dxmax DO BEGIN
  {computes seed dispersal along x file "east" of source}
 r := x-xcoord;
  numseeds := mult*(exp((alpha-1)*ln(r))*exp(r/beta)/denom);
   {determines number of seeds at distance r from a gamma distribution with
    parameters given by alpha, beta in the CONST definition above.}
  IF numseeds>1 THEN BEGIN
   j := (x-1)*50 + ycoord + (bankyr-1)*2500;
   xycell := Ptr(segadd + (j-1), offadd + 8*(j-1));
   WITH xycell^ DO specbank[specno] := 1;
  END;
 END;
END; {dispersal}


PROCEDURE germinate; {germinate seeds from the seedbank in "spring"}

LABEL NEXTCELL;

VAR
r1, r2, seedyear : integer;
seedarray : ARRAY[1..3,1..10] of integer;

BEGIN {germinate}
writeln('...germinating');
 FOR xcoord := 1 to max DO BEGIN
  FOR ycoord := 1 to max DO BEGIN
   IF universe[year,xcoord,ycoord,month] = 0 THEN BEGIN
   {if the cell is empty, then see if there are seeds in the seedbank}
    FOR seedyear := 1 to 3 DO BEGIN
     FOR specno := 1 to 10 DO BEGIN
      seedarray[seedyear,specno] := bankvalue(xcoord,ycoord,specno,seedyear);
     END;
    END;
    {choose which seed germinates}
     FOR seedyear := 1 to 3 DO BEGIN
      FOR specno := 1 to 10 DO BEGIN
       {check for flooding conditions. Remove comment braces to use.}
      CASE flood[xcoord,ycoord,((month-1) mod 2)+1] OF
        1..10 : BEGIN
         IF (species[specno,10]='D ') OR (species[specno,10]='B ') THEN BEGIN
          IF (seedarray[seedyear,specno] = 1) AND (species[specno,1] = 'A ')
           THEN BEGIN
            universe[year,xcoord,ycoord,month] := specno;
            goto NEXTCELL;
           END;
         END;
        END;
        ELSE BEGIN
         IF (species[specno,10]='U ') OR (species[specno,10]='B ') THEN BEGIN
          IF (seedarray[seedyear,specno] = 1) AND (species[specno,1] = 'A ')
           THEN BEGIN
            universe[year,xcoord,ycoord,month] := specno;
            goto NEXTCELL;
           END;
         END;
        END;
       END;    {Case section ends}
      END;
     END;
     m := 0;
     REPEAT
      r1 := Random(3)+1;
      r2 := Random(10)+1;
      CASE flood[xcoord,ycoord,((month-1) mod 2)+1] OF
       1..10 : BEGIN
        IF (species[r2,10]='D ') OR (species[r2,10]='B ') THEN BEGIN
         IF seedarray[r1,r2] = 1 THEN universe[year,xcoord,ycoord,month] := r2;
         m := m+1;
        END;
       END;
       ELSE BEGIN
        IF (species[r2,10]='U ') OR (species[r2,10]='B ') THEN BEGIN
         IF seedarray[r1,r2] = 1 THEN universe[year,xcoord,ycoord,month] := r2;
         m := m+1;
        END;
       END;
      END;      {case of flood}
     UNTIL (universe[year,xcoord,ycoord,month]>0) OR (m>50);
     NEXTCELL:
      BEGIN
       j := xcoord*50 + ycoord + (r1-1)*2500;
       xycell := Ptr(segadd+(j-1), offadd+8*(j-1));
       WITH xycell^ DO specbank[r2] := 0;
      END; {end germination}
   END;
  END;
 END;
END; {germinate}


PROCEDURE anngrow; {grow routine for annuals}
{Description: Annuals do not "grow" in that they can only occupy one cell of
 the wetland universe. Annuals always produce seeds (i.e. PROCEDURE dispersal
 is always called) in the season following that in which their "growth" is
 defined as maximal (i.e. species[specno,5]). Death occurs with fixed
 probability (.1) in any season. All annuals die in the winter.}
VAR
 growstring : string[11];
BEGIN
 growstring := ' S E F W C ';
 growseason := Pos(species[specno,5],growstring) div 2;
  IF (month = growseason + 1) THEN BEGIN
    dispersal;
  END;
  IF (month<4) or (month = growseason + 2) THEN
   universe[year,xcoord,ycoord,month+1] := universe[year,xcoord,ycoord,month];
  IF (month = 4) THEN universe[year,xcoord,ycoord,month] := 0;
END; {anngrow}



PROCEDURE pergrow; {grow routine for perennials}
{Description: There are three morphological types of perennials - rhizomatous,
 bulbous, and tuberous. Each grows in a different way, and according to whether
 it is a Guerrilla or Phalanx directional grower. For rhizomatous plants, first
 a direction of growth is chosen from one of 8 directions (N,S,E,W,NE,NW,SE,SW).
 If it is a phalanx grower, then the plant colonizes one cell in the chosen
 direction +/- one direction. If it is a guerrilla grower, then the plant
 colonizes one cell three cells distant from the cell of origin in the chosen
 direction. For bulbous plants, in the season of maximal growth, new bulbs
 occupy all adjacent squares (phalanx morphology) or two contiguous, adjacent
 squares (guerrillas). For tuberous plants, new shoots occupy all adjacent
 squares (phalanx) or three contigous squares in one direction (guerrilla). All
 perennials rarely produce seeds, and dispersal is called with probability
 0.001. Death occurs at a constant rate of .05/season of growth.}

VAR
 army,morph,speed,direction : integer;
 growstring : string[11];
 morphtype : string[7];
 speedstring : string[5];
 armystring : string[5];

PROCEDURE growguerrilla;
           BEGIN {guerrilla species}
            universe[year,xcoord,ycoord,month+1] :=
             universe[year,xcoord,ycoord,month];
            direction := Random(8) + 1;
             CASE speed OF
              1 : BEGIN {slow growers}
                     CASE direction OF
                      1: BEGIN
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                         END;
                      2: BEGIN
                          if (ycoord - 1 > 0) and (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                         END;
                      3: BEGIN
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                         END;
                      4: BEGIN
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                         END;
                      5: BEGIN
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                         END;
                      6: BEGIN
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                         END;
                      7: BEGIN
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                         END;
                      8: BEGIN
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                         END;
                     END;
                    END;
              2 : BEGIN {fast growers}
                     CASE direction OF
                      1: BEGIN
                          if (morph=3) and
                           (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (morph>1) and
                           (ycoord - 2 > 0) and
                           (universe[year,xcoord,ycoord-2,month] = 0) then
                           universe[year,xcoord,ycoord-2,month+1] := specno;
                          if (ycoord - 3 > 0) and
                           (universe[year,xcoord,ycoord-3,month] = 0) then
                           universe[year,xcoord,ycoord-3,month+1] := specno;
                         END;
                      2: BEGIN
                          if (morph=3) and
                           (ycoord - 1 > 0) and (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (morph>1) and
                           (ycoord - 2 > 0) and (xcoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord-2,month] = 0) then
                           universe[year,xcoord+2,ycoord-2,month+1] := specno;
                          if (ycoord - 3 > 0) and (xcoord + 3 < 51) and
                           (universe[year,xcoord+3,ycoord-3,month] = 0) then
                           universe[year,xcoord+3,ycoord-3,month+1] := specno;
                         END;
                      3: BEGIN
                          if (morph=3) and
                           (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (morph>1) and
                           (xcoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord,month] = 0) then
                           universe[year,xcoord+2,ycoord,month+1] := specno;
                          if (xcoord + 3 < 51) and
                           (universe[year,xcoord+3,ycoord,month] = 0) then
                           universe[year,xcoord+3,ycoord,month+1] := specno;
                         END;
                      4: BEGIN
                          if (morph=3) and
                           (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (morph>1) and
                           (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord+2,month] = 0) then
                           universe[year,xcoord+2,ycoord+2,month+1] := specno;
                          if (xcoord + 3 < 51) and (ycoord + 3 < 51) and
                           (universe[year,xcoord+3,ycoord+3,month] = 0) then
                           universe[year,xcoord+3,ycoord+3,month+1] := specno;
                         END;
                      5: BEGIN
                          if (morph=3) and
                           (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (morph>1) and
                           (ycoord + 2 < 51) and
                           (universe[year,xcoord,ycoord+2,month] = 0) then
                           universe[year,xcoord,ycoord+2,month+1] := specno;
                          if (ycoord + 3 < 51) and
                           (universe[year,xcoord,ycoord+3,month] = 0) then
                           universe[year,xcoord,ycoord+3,month+1] := specno;
                         END;
                      6: BEGIN
                          if (morph=3) and
                           (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (morph>1) and
                           (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                           (universe[year,xcoord-2,ycoord+2,month] = 0) then
                           universe[year,xcoord-2,ycoord+2,month+1] := specno;
                          if (xcoord - 3 > 0) and (ycoord + 3 < 51) and
                           (universe[year,xcoord-3,ycoord+3,month] = 0) then
                           universe[year,xcoord-3,ycoord+3,month+1] := specno;
                         END;
                      7: BEGIN
                          if (morph=3) and
                           (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (morph>1) and
                           (xcoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord,month] = 0) then
                           universe[year,xcoord-2,ycoord,month+1] := specno;
                          if (xcoord - 3 > 0) and
                           (universe[year,xcoord-3,ycoord,month] = 0) then
                           universe[year,xcoord-3,ycoord,month+1] := specno;
                         END;
                      8: BEGIN
                          if (morph=3) and
                           (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (morph>1) and
                           (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord-2,month] = 0) then
                           universe[year,xcoord-2,ycoord-2,month+1] := specno;
                          if (xcoord - 3 > 0) and (ycoord - 3 > 0) and
                           (universe[year,xcoord-3,ycoord-3,month] = 0) then
                           universe[year,xcoord-3,ycoord-3,month+1] := specno;
                         END;
                     END;
                    END;
             END;
          END; {growguerrilla}

BEGIN {pergrow}
 growstring := ' S E F W C ';
 morphtype := ' R B T ';
 speedstring := ' S F ';
 armystring := ' P G ';
 growseason := Pos(species[specno,5],growstring) div 2;
 morph := Pos(species[specno,2],morphtype) div 2;
 speed := Pos(species[specno,4],speedstring) div 2;
 army := Pos(species[specno,3],armystring) div 2;
 CASE morph OF
  1: BEGIN {if perennial and rhizomatous}
        IF (month = growseason) or (growseason = 5) THEN
         BEGIN {grow the plant if it's the maximal season of growth}
          IF species[specno,3] = 'P ' THEN
            BEGIN {phalanx species}
            universe[year,xcoord,ycoord,month+1] :=
             universe[year,xcoord,ycoord,month];
            direction := Random(8) + 1;
            CASE speed OF
             1 : BEGIN {slow growers}
                     CASE direction OF
                      1: BEGIN
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                         END;
                      2: BEGIN
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                         END;
                      3: BEGIN
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                         END;
                      4: BEGIN
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                         END;
                      5: BEGIN
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                         END;
                      6: BEGIN
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                         END;
                      7: BEGIN
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                         END;
                      8: BEGIN
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                         END;
                     END;
                    END;
              2 : BEGIN {fast growers}
                     CASE direction OF
                      1: BEGIN
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (ycoord - 2 > 0) and
                           (universe[year,xcoord,ycoord-2,month] = 0) then
                           universe[year,xcoord,ycoord-2,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord-2,month] = 0) then
                           universe[year,xcoord-2,ycoord-2,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord - 2 > 0) and
                           (universe[year,xcoord+2,ycoord-2,month] = 0) then
                           universe[year,xcoord+2,ycoord-2,month+1] := specno;
                         END;
                      2: BEGIN
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (ycoord - 2 > 0) and
                           (universe[year,xcoord,ycoord-2,month] = 0) then
                           universe[year,xcoord,ycoord-2,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord - 2 > 0) and
                           (universe[year,xcoord+1,ycoord-2,month] = 0) then
                           universe[year,xcoord+1,ycoord-2,month+1] := specno;
                          if (xcoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord,month] = 0) then
                           universe[year,xcoord+2,ycoord,month+1] := specno;
                         END;
                      3: BEGIN
                          if (xcoord + 1 < 51) and (ycoord - 1 > 0) and
                           (universe[year,xcoord+1,ycoord-1,month] = 0) then
                           universe[year,xcoord+1,ycoord-1,month+1] := specno;
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord - 2 > 0) and
                           (universe[year,xcoord+2,ycoord-2,month] = 0) then
                           universe[year,xcoord+2,ycoord-2,month+1] := specno;
                          if (xcoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord,month] = 0) then
                           universe[year,xcoord+2,ycoord,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord+2,month] = 0) then
                           universe[year,xcoord+2,ycoord+2,month+1] := specno;
                         END;
                      4: BEGIN
                          if (xcoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord,month] = 0) then
                           universe[year,xcoord+1,ycoord,month+1] := specno;
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (xcoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord,month] = 0) then
                           universe[year,xcoord+2,ycoord,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord+2,month] = 0) then
                           universe[year,xcoord+2,ycoord+2,month+1] := specno;
                          if (ycoord + 2 < 51) and
                           (universe[year,xcoord,ycoord+2,month] = 0) then
                           universe[year,xcoord,ycoord+2,month+1] := specno;
                         END;
                      5: BEGIN
                          if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                           (universe[year,xcoord+1,ycoord+1,month] = 0) then
                           universe[year,xcoord+1,ycoord+1,month+1] := specno;
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                           (universe[year,xcoord+2,ycoord+2,month] = 0) then
                           universe[year,xcoord+2,ycoord+2,month+1] := specno;
                          if (ycoord + 2 < 51) and
                           (universe[year,xcoord,ycoord+2,month] = 0) then
                           universe[year,xcoord,ycoord+2,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                           (universe[year,xcoord-2,ycoord+2,month] = 0) then
                           universe[year,xcoord-2,ycoord+2,month+1] := specno;
                         END;
                      6: BEGIN
                          if (ycoord + 1 < 51) and
                           (universe[year,xcoord,ycoord+1,month] = 0) then
                           universe[year,xcoord,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (ycoord + 2 < 51) and
                           (universe[year,xcoord,ycoord+2,month] = 0) then
                           universe[year,xcoord,ycoord+2,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                           (universe[year,xcoord-2,ycoord+2,month] = 0) then
                           universe[year,xcoord-2,ycoord+2,month+1] := specno;
                          if (xcoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord,month] = 0) then
                           universe[year,xcoord-2,ycoord,month+1] := specno;
                         END;
                      7: BEGIN
                          if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                           (universe[year,xcoord-1,ycoord+1,month] = 0) then
                           universe[year,xcoord-1,ycoord+1,month+1] := specno;
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                           (universe[year,xcoord-2,ycoord+2,month] = 0) then
                           universe[year,xcoord-2,ycoord+2,month+1] := specno;
                          if (xcoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord,month] = 0) then
                           universe[year,xcoord-2,ycoord,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord-2,month] = 0) then
                           universe[year,xcoord-2,ycoord-2,month+1] := specno;
                         END;
                      8: BEGIN
                          if (xcoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                           (universe[year,xcoord-1,ycoord-1,month] = 0) then
                           universe[year,xcoord-1,ycoord-1,month+1] := specno;
                          if (ycoord - 1 > 0) and
                           (universe[year,xcoord,ycoord-1,month] = 0) then
                           universe[year,xcoord,ycoord-1,month+1] := specno;
                          if (xcoord - 2 > 0) and
                           (universe[year,xcoord-1,ycoord,month] = 0) then
                           universe[year,xcoord-1,ycoord,month+1] := specno;
                          if (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                           (universe[year,xcoord-2,ycoord-2,month] = 0) then
                           universe[year,xcoord-2,ycoord-2,month+1] := specno;
                          if (ycoord - 2 > 0) and
                           (universe[year,xcoord,ycoord-2,month] = 0) then
                           universe[year,xcoord,ycoord-2,month+1] := specno;
                         END;
                     END;
                    END;
            END;
          END
           ELSE IF species[specno,3] = 'G ' THEN growguerrilla;
       END {if growseason}
         ELSE
          BEGIN
           if (month=(growseason+1)) and (Random(1001)<2) then dispersal;
           if (month<4) and (month<>growseason) and (growseason<5) then
             universe[year,xcoord,ycoord,month+1] :=
              universe[year,xcoord,ycoord,month];
          END;
       END; {if rhizomatous}
  2: BEGIN {if perennial and bulbous}
        IF (growseason = month) or (growseason = 5) THEN
         BEGIN {grow the plant if it's the maximal season of growth}
          IF species[specno,3] = 'P ' THEN
           BEGIN {phalanx species}
            universe[year,xcoord,ycoord,month+1] :=
             universe[year,xcoord,ycoord,month];
            CASE speed OF
             1 : BEGIN {slow growers}
                    if (ycoord - 1 > 0) and
                     (universe[year,xcoord,ycoord-1,month] = 0) then
                     universe[year,xcoord,ycoord-1,month+1] := specno;
                    if (ycoord - 1 < 0) and (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord-1,month] = 0) then
                     universe[year,xcoord+1,ycoord-1,month+1] := specno;
                    if (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord,month] = 0) then
                     universe[year,xcoord+1,ycoord,month+1] := specno;
                    if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord+1,month] = 0) then
                     universe[year,xcoord+1,ycoord+1,month+1] := specno;
                    if (ycoord + 1 < 51) and
                     (universe[year,xcoord,ycoord+1,month] = 0) then
                     universe[year,xcoord,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                     (universe[year,xcoord-1,ycoord+1,month] = 0) then
                     universe[year,xcoord-1,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord,month] = 0) then
                     universe[year,xcoord-1,ycoord,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord-1,month] = 0) then
                     universe[year,xcoord-1,ycoord-1,month+1] := specno;
                   END;
             2 : BEGIN {fast growers}
                    if (ycoord - 1 > 0) and
                     (universe[year,xcoord,ycoord-1,month] = 0) then
                     universe[year,xcoord,ycoord-1,month+1] := specno;
                    if (ycoord - 1 < 0) and (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord-1,month] = 0) then
                     universe[year,xcoord+1,ycoord-1,month+1] := specno;
                    if (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord,month] = 0) then
                     universe[year,xcoord+1,ycoord,month+1] := specno;
                    if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord+1,month] = 0) then
                     universe[year,xcoord+1,ycoord+1,month+1] := specno;
                    if (ycoord + 1 < 51) and
                     (universe[year,xcoord,ycoord+1,month] = 0) then
                     universe[year,xcoord,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                     (universe[year,xcoord-1,ycoord+1,month] = 0) then
                     universe[year,xcoord-1,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord,month] = 0) then
                     universe[year,xcoord-1,ycoord,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord-1,month] = 0) then
                     universe[year,xcoord-1,ycoord-1,month+1] := specno;
                    if (ycoord - 2 > 0) and
                     (universe[year,xcoord,ycoord-2,month] = 0) then
                     universe[year,xcoord,ycoord-2,month+1] := specno;
                    if (ycoord - 2 < 0) and (xcoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord-2,month] = 0) then
                     universe[year,xcoord+2,ycoord-2,month+1] := specno;
                    if (xcoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord,month] = 0) then
                     universe[year,xcoord+2,ycoord,month+1] := specno;
                    if (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord+2,month] = 0) then
                     universe[year,xcoord+2,ycoord+2,month+1] := specno;
                    if (ycoord + 2 < 51) and
                     (universe[year,xcoord,ycoord+2,month] = 0) then
                     universe[year,xcoord,ycoord+2,month+1] := specno;
                    if (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                     (universe[year,xcoord-2,ycoord+2,month] = 0) then
                     universe[year,xcoord-2,ycoord+2,month+1] := specno;
                    if (xcoord - 2 > 0) and
                     (universe[year,xcoord-2,ycoord,month] = 0) then
                     universe[year,xcoord-2,ycoord,month+1] := specno;
                    if (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                     (universe[year,xcoord-2,ycoord-2,month] = 0) then
                     universe[year,xcoord-2,ycoord-2,month+1] := specno;
                   END;
            END;
           END
           ELSE IF species[specno,3] = 'G ' THEN growguerrilla;
         END {if growseason}
         ELSE
          BEGIN
           if (month=(growseason+1)) and (Random(1001)<2) then dispersal;
           if (month<4)  and (month <> growseason) and (growseason<5) then
             universe[year,xcoord,ycoord,month+1] :=
              universe[year,xcoord,ycoord,month];
          END;
       END;  {if bulbous}
  3: BEGIN {if perennial and tuberous}
        IF (growseason = month) or (growseason = 5) THEN
         BEGIN {grow the plant if it's the maximal season of growth}
          IF species[specno,3] = 'P ' THEN
           BEGIN {phalanx species}
            universe[year,xcoord,ycoord,month+1] :=
             universe[year,xcoord,ycoord,month];
            CASE speed OF
             1 : BEGIN {slow growers}
                    if (ycoord - 1 > 0) and
                     (universe[year,xcoord,ycoord-1,month] = 0) then
                     universe[year,xcoord,ycoord-1,month+1] := specno;
                    if (ycoord - 1 < 0) and (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord-1,month] = 0) then
                     universe[year,xcoord+1,ycoord-1,month+1] := specno;
                    if (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord,month] = 0) then
                     universe[year,xcoord+1,ycoord,month+1] := specno;
                    if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord+1,month] = 0) then
                     universe[year,xcoord+1,ycoord+1,month+1] := specno;
                    if (ycoord + 1 < 51) and
                     (universe[year,xcoord,ycoord+1,month] = 0) then
                     universe[year,xcoord,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                     (universe[year,xcoord-1,ycoord+1,month] = 0) then
                     universe[year,xcoord-1,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord,month] = 0) then
                     universe[year,xcoord-1,ycoord,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord-1,month] = 0) then
                     universe[year,xcoord-1,ycoord-1,month+1] := specno;
                   END;
             2 : BEGIN {fast growers}
                    if (ycoord - 1 > 0) and
                     (universe[year,xcoord,ycoord-1,month] = 0) then
                     universe[year,xcoord,ycoord-1,month+1] := specno;
                    if (ycoord - 1 < 0) and (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord-1,month] = 0) then
                     universe[year,xcoord+1,ycoord-1,month+1] := specno;
                    if (xcoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord,month] = 0) then
                     universe[year,xcoord+1,ycoord,month+1] := specno;
                    if (xcoord + 1 < 51) and (ycoord + 1 < 51) and
                     (universe[year,xcoord+1,ycoord+1,month] = 0) then
                     universe[year,xcoord+1,ycoord+1,month+1] := specno;
                    if (ycoord + 1 < 51) and
                     (universe[year,xcoord,ycoord+1,month] = 0) then
                     universe[year,xcoord,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord + 1 < 51) and
                     (universe[year,xcoord-1,ycoord+1,month] = 0) then
                     universe[year,xcoord-1,ycoord+1,month+1] := specno;
                    if (xcoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord,month] = 0) then
                     universe[year,xcoord-1,ycoord,month+1] := specno;
                    if (xcoord - 1 > 0) and (ycoord - 1 > 0) and
                     (universe[year,xcoord-1,ycoord-1,month] = 0) then
                     universe[year,xcoord-1,ycoord-1,month+1] := specno;
                    if (ycoord - 2 > 0) and
                     (universe[year,xcoord,ycoord-2,month] = 0) then
                     universe[year,xcoord,ycoord-2,month+1] := specno;
                    if (ycoord - 2 < 0) and (xcoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord-2,month] = 0) then
                     universe[year,xcoord+2,ycoord-2,month+1] := specno;
                    if (xcoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord,month] = 0) then
                     universe[year,xcoord+2,ycoord,month+1] := specno;
                    if (xcoord + 2 < 51) and (ycoord + 2 < 51) and
                     (universe[year,xcoord+2,ycoord+2,month] = 0) then
                     universe[year,xcoord+2,ycoord+2,month+1] := specno;
                    if (ycoord + 2 < 51) and
                     (universe[year,xcoord,ycoord+2,month] = 0) then
                     universe[year,xcoord,ycoord+2,month+1] := specno;
                    if (xcoord - 2 > 0) and (ycoord + 2 < 51) and
                     (universe[year,xcoord-2,ycoord+2,month] = 0) then
                     universe[year,xcoord-2,ycoord+2,month+1] := specno;
                    if (xcoord - 2 > 0) and
                     (universe[year,xcoord-2,ycoord,month] = 0) then
                     universe[year,xcoord-2,ycoord,month+1] := specno;
                    if (xcoord - 2 > 0) and (ycoord - 2 > 0) and
                     (universe[year,xcoord-2,ycoord-2,month] = 0) then
                     universe[year,xcoord-2,ycoord-2,month+1] := specno;
                   END;
            END;
           END
           ELSE IF species[specno,3] = 'G ' THEN growguerrilla;
         END {if growseason}
         ELSE
          BEGIN
           if (month=(growseason+1)) and (Random(1001)<2) then dispersal;
           if (month<4)  and (month <>growseason) and (growseason<5) then
             universe[year,xcoord,ycoord,month+1] :=
              universe[year,xcoord,ycoord,month];
          END;
       END; {if tuberous}
 END; {case of morph}
END; {pergrow}



PROCEDURE growdriv; {driver for growth routines}

BEGIN {growdriv}
 FOR runtime := 1 to topyr do BEGIN
  FOR year := 1 to maxyr do BEGIN
   writeln('...growing year ',(year + ((runtime-1)*3)):3);
   FOR month := 1 to 4 do BEGIN
   IF ((month = 1) and (year + (runtime-1) > 1)) THEN
    BEGIN
     FOR xcoord := 1 to max DO BEGIN
     FOR ycoord := 1 to max DO BEGIN
      IF ((runtime =1) OR (year > 1)) THEN
       universe[year,xcoord,ycoord,month] := universe[year-1,xcoord,ycoord,4]
      ELSE IF ((runtime>1) AND (year=1)) THEN
       universe[year,xcoord,ycoord,month] := universe[3,xcoord,ycoord,4];
     END;
     END;
      germinate;
      updatebank;
    END;
    FOR xcoord := 1 to max do BEGIN
    FOR ycoord := 1 to max do BEGIN
     if universe[year,xcoord,ycoord,month] >0 then BEGIN
      specno := universe[year,xcoord,ycoord,month];
      if species[specno,1] = 'A ' then
        anngrow {call grow routine for annuals}
       else
      pergrow {call grow routine for perennials};
     END;
    END;
    END;
    death;
    writeln('going to printout');
    printout;
    waterlevel;
   END;
  END;
 END;
END;{growdriv}

