{Introduction and copyright for program WETLAND. Contains procedure INTRODUCE}

{DEBUGGED AND RUNNING 09-DEC-1986}
{AMENDED}


OVERLAY PROCEDURE introduce; {introduce the program}
  BEGIN
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln;
   writeln('               W     W  EEEEE  TTTTT L      AAAAA  N   N  DDD');
   writeln('               W  W  W  E        T   L      A   A  NN  N  D  D');
   writeln('               W W W W  EEE      T   L      AAAAA  N N N  D   D');
   writeln('               WW   WW  E        T   L      A   A  N  NN  D  D');
   writeln('               W     W  EEEEE    T   LLLLL  A   A  N   N  DDD');
   writeln;
   writeln('                                 Version 1.0                  ');
   writeln;
   writeln('                                     by                       ');
   writeln;
   writeln('                                Aaron M. Ellison              ');
   writeln('                          Ecosystems Research Center          ');
   writeln('                               Cornell University             ');
   writeln('                             Ithaca, New York 14853           ');
   writeln;
   writeln('                               Copyright (c) 1987             ');
   writeln;
   writeln('===>Press any key to begin:');
    REPEAT
     delay(1);
    UNTIL KeyPressed;
    read(kbd,go);
    ClrScr;
    BEGIN
     writeln;
     writeln('   Welcome to WETLAND, a computer simulation of wetland plant');
     writeln('communities'' response(s) to flooding.  This program was written');
     writeln('and developed by Aaron M. Ellison, Ecosystems Research Center,');
     writeln('Cornell University, Ithaca, New York, 14853.');
     writeln('The random number generator routines used in this program are');
     writeln('liccensed portions of TURBO_RAND, Copyright 1986 Loesl, Ltd.,');
     writeln('Pennellville, New York, 13132.');
     writeln('   The wetland simulated by WETLAND is a 50x50 square universe.');
     writeln('In this wetland, species grow, die, potentially interact, and re-');
     writeln('produce. Only one species can occupy any given square at a given');
     writeln('time, but seeds of up to ',maxsp,' species can be in the seed bank of any');
     writeln('given square at any time. Seeds can remain in the seed bank for up');
     writeln('to three years, but will always germinate if conditions are right.');
     writeln('   Up to ',maxsp, ' species may inhabit the wetland, and either you');
     writeln('can determine their characteristics, or the computer can define them');
     writeln('If you do not understand any of the characteristics used, enter ''HELP''');
     writeln('when prompted for a characteristic or refer to the user''s guide.');
    END;
END;



{Initialize procedure for program WETLAND. Contains the procedure INTIALIZE}

{Copyright 1987 by Aaron M. Ellison}

{DEBUGGED AND RUNNING 01-DEC-1986}

OVERLAY PROCEDURE initialize;

 {This procedure initializes important variables and arrays.
  The matrices holding the wetland (50x50x12 = max x maxyr x 4)
  and the species description (10 x13 = maxsp x maxyr) are set
  to 0. These two matrices are stored in memory.  Because of the
  64K memory limitation of TurboPascal, the seedbank is constructed
  as a record with 7500 10-element (=maxsp) variables.  The
  seedbank is stored in the heap, and is addressed using pointers.
  Critical functions for correct usage of the seedbank are the
  Ofs and Seg functions (built in to TurboPascal) and the BankValue
  function written into this program.  See the TurboPascal manual
  for more information on stacks, heaps, and pointers.}

  BEGIN {initialize}
   write(' For how many years would you like to run the simulation ====> ');
   readln(topyear);
    topyr := topyear div 3;
   compugo := 'C';
   startflood := 1;
   for i := 1 to maxsp do {set species description matrix to blanks}
    BEGIN for j := 1 to maxchar do
     species[i,j] := '  '
    END;
   for i := 1 to maxyr do {set wetland matrix to 0}
    BEGIN  for j := 1 to max do
      BEGIN  for k := 1 to max do
        BEGIN for l := 1 to 4 do
          universe[i,j,k,l] :=0;
        END;
      END;
    END;
   for i := 1 to max do
    BEGIN for j :=1 to max do
     BEGIN for k :=1 to 2 do
      flood[i,j,k] := 0;
     END;
    END;
   for j := 1 to 7500 do {set seedbank array to 0}
    BEGIN
     NEW(xycell);
     if j = 1 then heaptop := xycell;
     for i := 1 to 10 do
      BEGIN
       with xycell^ do specbank[i] := 0;
      END;
    END;
    segadd := seg(heaptop^);
    offadd := ofs(heaptop^);
  END;


OVERLAY FUNCTION bankvalue (var xcoord,ycoord,specno,year : integer) : integer;

{For use with program WETLAND}

{This function returns the number of seeds of species <specno> in the
 seedbank at coordinates <xcoord,ycoord,year> of the wetland universe
 matrix.}

{Copyright 1987 by Aaron M. Ellison}

{DEBUGGED AND RUNNING 15-DEC-1986}

BEGIN
 j := (xcoord-1)*50 + ycoord + (year-1)*2500;
 xycell := Ptr(segadd + (j-1), offadd + 8*(j-1));
 bankvalue := xycell^.specbank[specno];
END;


OVERLAY FUNCTION factorial (value: integer) : real;

BEGIN
 if value = 0 then factorial := 1
 else factorial := value*factorial(value-1);
END;



OVERLAY PROCEDURE updatebank;

{For use with program WETLAND. Updates the seedbank on a yearly basis to
 drop seeds that have been in the bank beyond their lifespan.}

{Copyright 1987 by Aaron M. Ellison}

{COMPILED AND RUNNING 3-APR-1987}

VAR
 nxycell, oxycell : ^bankcell;

BEGIN {updatebank}
 FOR j := 1 to 5000 DO
  BEGIN
   FOR specno := 1 to nspecies DO
    BEGIN
     nxycell := Ptr(segadd + (j-1), offadd + 8*(j-1));
     oxycell := Ptr(segadd + (j+2499), offadd + 8*(j+2499));
     nxycell^.specbank[specno] := oxycell^.specbank[specno];
    END;
  END;
END; {updatebank}


{Copyright 1987 by Aaron M. Ellison}

{Debugged and Running 16-FEB-1987}

OVERLAY PROCEDURE youseed {user seeds the wetland universe};
VAR goseed : string[1];
    ichng : byte;
LABEL BOZO, BOZO1;
 BEGIN {youseed}
  writeln;
  writeln('   To define the species starting conditions, you must enter the');
  writeln(' location in the wetland that each of the ',nspecies,' species');
  writeln(' occupies. The wetland is a 50x50 grid, and each species initially');
  writeln(' is put into one space on the grid. Each space is defined by a set');
  writeln(' of x,y coordinates. When asked for the starting location, please');
  writeln(' enter the x and y coordinates SEPARATED BY A SPACE (not a comma).');
  writeln;
   goseed := 'Y';
  for i := 1 to nspecies do
   BEGIN
    xstart[i] := 0;
    ystart[i] := 0;
   END;
  for i := 1 to nspecies do
   BEGIN
    writeln;
    writeln('The definition for species ',i,' is:');
    showchars(i);
    BOZO:REPEAT
          writeln;
          write('Enter the starting location for species ', i, ' ====> ');
          readln(xstart[i],ystart[i]);
         UNTIL (xstart[i] IN [1..50]) AND (ystart[i] IN [1..50]);
         IF universe[1,xstart[i],ystart[i],1] = 0 THEN
          universe[1,xstart[i],ystart[i],1] := i
         ELSE
          BEGIN
           writeln;
           writeln('Location is already occupied');
           GOTO BOZO
          END;
   END;
  REPEAT
   writeln('  The species starting locations are: ');
   writeln;
   writeln('Species      x       y   ');
   for i := 1 to nspecies do
    writeln(i:4,xstart[i]:10,ystart[i]:8);
   writeln;
   write('  Do you want to change any of the starting locations (y/n): ');
   readln(goseed);
   if upcase(goseed) = 'Y' then
    BEGIN
    write('  Which species number? ');
    readln(ichng);
    BOZO1:REPEAT
          writeln;
          write('  Enter the new starting location for species ',ichng,' ===> ');
          readln(xstart[ichng],ystart[ichng]);
         UNTIL (xstart[ichng] IN [1..50]) AND (ystart[ichng] IN [1..50]);
         IF universe[1,xstart[ichng],ystart[ichng],1] = 0 THEN
          universe[1,xstart[ichng],ystart[ichng],1] := i
         ELSE
          BEGIN
           writeln;
           writeln('  Location is already occupied, try again');
           GOTO BOZO1
          END;
    END;
  UNTIL upcase(goseed) = 'N';
END {youseed};


OVERLAY PROCEDURE iseed {computer sets the starting conditions};
 VAR
   ranset : ARRAY[1..20] of integer;
   seed_ : arrspec;
 BEGIN {iseed}
 writeln('running iseed');
  setarrsp(ranset[1], ranset[20], sizeof(ranset[1]), seed_, serr);
   IF (serr <>0) THEN
    BEGIN {error handling procedure}
     writeln(' Bombing in iseed...user must set starting conditions');
     youseed;
     exit;
    END
   ELSE
    write(' Enter random seed ====> ');
    readln(ranseed);
    arnirswo(seed_, ranseed, 1, 50); {generate an array of 20 random numbers}
  FOR i := 1 TO nspecies DO
   universe[1,ranset[i],ranset[i+10],1] := i;
  writeln('  The species starting locations are: ');
  writeln;
  writeln('Species      x       y');
  FOR i := 1 TO nspecies DO
   writeln(i:4,ranset[i]:10,ranset[i+10]:8);
   writeln;
   writeln('===>Press any key to continue');
    REPEAT
     delay(1);
    UNTIL KeyPressed;
    read(kbd,go);
    writeln('ending iseed, going to grow');
 END {iseed};

OVERLAY PROCEDURE death; {kills plants}
BEGIN {death}
 FOR xcoord := 1 to max do BEGIN
 FOR ycoord := 1 to max do BEGIN
  if universe[year,xcoord,ycoord,month+1] >0 then BEGIN
   specno := universe[year,xcoord,ycoord,month+1];
   if (species[specno,1] = 'P ') and (Random(101) < 2) then
      universe[year,xcoord,ycoord,month+1] := 0;
   if (species[specno,1] = 'A ') and (Random(101) < 2) then
      universe[year,xcoord,ycoord,month+1] := 0;
  END;
 END;
 END;
END; {death}


{Procedure to regulate water levels in marsh. Uses array[1..max,1..max,1..2]
 called flood}

{For use with Program WETLAND}

{Copyright 1986, 1987 by Aaron M. Ellison}

{DEBUGGED AND RUNNING xx MAY 1987}
{AMENDED          }

PROCEDURE waterlevel;

VAR
 ncount,ocount : integer;

BEGIN
 IF empty <10.0 THEN BEGIN
  IF startflood = 1 THEN BEGIN
   flood[Random(50)+1,Random(50)+1,((month-1) mod 2)+1] := 1;
   startflood := 0;
  END;
  ncount := (month mod 2)+1;
  ocount := ((month-1) mod 2)+1;
  FOR j := 1 to max DO BEGIN
   FOR k := 1 to max DO BEGIN
    CASE flood[j,k,ocount] OF
     0      : BEGIN
               IF ((j-1>0) AND (flood[j-1,k,ocount]=1)) OR
               ((k-1>0) AND (flood[j,k-1,ocount]=1)) OR
               ((j+1<51) AND (flood[j+1,k,ocount]=1)) OR
               ((k+1<51) AND (flood[j,k+1,ocount]=1))
                THEN flood[j,k,ncount]:=1;
              END;
     1..255 : BEGIN
               IF flood[j,k,ocount]<256 THEN
                flood[j,k,ncount] := flood[j,k,ocount]+1
               ELSE
                flood[j,k,ncount] := flood[j,k,ocount]+1;
              END;
     END;
    END;
   END;
  END;
END;

